################################################################################
# This script is an license sector listbox for ArcGIS Pro
# 
#  Script name: ArcGIS_Pro_License_Selector.pyw
#  Release 1.8
#  Written by Jeremy Saunders (jeremy@jhouseconsulting.com) 17th February 2020
#  Modified by Jeremy Saunders (jeremy@jhouseconsulting.com) 22nd February 2024
# 
################################################################################

# Set these variables

# Set the minutes that this window will stay open for before closing
# Set to 0 to disable the timer
MinutesBeforeClosing = 720

# Set to True to default the license selection to Basic. This helps to reset the
# licensing after each use. Setting it to False will set it to whatever was last
# selected.
AlwaysDefaultToBasic = True

# Set to True to enable the Reset Extensions checkbox. This will clear any
# Extensions that were previously selected, which helps to reset the licensing
# after each use.
AlwaysResetExtensions = True

# Set to True to give the users the ability to install extra Python Modules
InstallPythonModules = False

# This variable contains a list of extra Python modules to install
PythonModulesToInstall = ['pyodbc','PyYAML','Pillow']

# Set to True to give the users the ability to launch File Explorer
AddFileExplorer = True

# Set to True to enable screen resolution (width and height) output in pixels to
# assist with the geometry of the main window.
EnableScreenResolutionDebugging = False

################################################################################

import os, sys, subprocess
if sys.version_info[0] == 3:
    import winreg
    from tkinter import *
    from tkinter import messagebox
else:
    import _winreg as winreg
    from Tkinter import *
    import tkMessageBox

# Derive the installation location and ensure ArcGISPro.exe exists    
try:
    rk = winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE,r"SOFTWARE\ESRI\ArcGISPro")                        
    tuple = winreg.QueryValueEx(rk,"InstallDir")
    Path = tuple[0]                     
    winreg.CloseKey(rk)
except:
    print ("Registry key not accessible!")

try:
    Path
except NameError:
    print("Could not get the ArcGIS Pro InstallDir from the registry")
    sys.exit(0)

ArcGISProPath = Path + "bin\\ArcGISPro.exe"
exists = os.path.isfile(ArcGISProPath)
if not exists:
    print("The folder path for ArcGIS Pro does not exist")
    sys.exit(0)

# Set registry value
def set_reg(path, name, value):
    try:
        winreg.CreateKey(winreg.HKEY_CURRENT_USER, path)
        registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0, 
                                       winreg.KEY_WRITE)
        winreg.SetValueEx(registry_key, name, 0, winreg.REG_SZ, value)
        winreg.CloseKey(registry_key)
        return True
    except WindowsError:
        return False

# Get registry value
def get_reg(path, name):
    try:
        registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0,
                                       winreg.KEY_READ)
        value, regtype = winreg.QueryValueEx(registry_key, name)
        winreg.CloseKey(registry_key)
        return value
    except WindowsError:
        return None

# Delete all registry values under a key
def delete_all_reg_values(path):
    try:
        registry_key = winreg.OpenKey(winreg.HKEY_CURRENT_USER, path, 0,
                                       winreg.KEY_ALL_ACCESS)
        while True:
            try:
                value_name = winreg.EnumValue(registry_key, 0)[0]
                winreg.DeleteValue(registry_key, value_name)
            except OSError:
                # When EnumValue raises an exception, it means there are no more values
                break
        winreg.CloseKey(registry_key)
        return True
    except WindowsError:
        return False

path = r"Software\ESRI\ArcGISPro\Licensing"

# Use the %USERPROFILE% as the working folder
workingfolder = os.getenv('USERPROFILE')

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# To avoid the GUI from becoming unresponsive due to a "thread lock" issue, we can utilize the after()
# method of the tkinter window to schedule the window to close after a specified duration without
# blocking the main event loop. We define the close_after() function, which uses the after() method to
# schedule the close_window() function to be called after the specified delay. This approach allows the
# main event loop to continue running and keeps the GUI responsive. It is good practice to define these
# outside of the mainloop to ensure it's only called once and not with each iteration of the loop.

def close_window():
    root.destroy()

def close_after(delay):
    root.after(delay, close_window)

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

root = Tk()
# Preventing the root window to change size
root.resizable(0,0)
# Allowing the root window to change size
#root.resizable(True, True)
root.title("ArcGIS Pro License Selector")
root.iconbitmap(ArcGISProPath)

computername = os.environ['COMPUTERNAME']
l1 = Label(root, text='You are running this on ' + computername ,relief = FLAT, fg = 'gray40' , anchor = "w", font = ('Arial', 8, 'bold'))
l1.grid()

l2 = Label(root, text='Choose the type of license:' ,relief = FLAT, fg = 'gray20' , anchor = "w", font = ('Arial', 10, 'bold'), width = 25)
l2.grid()
choices = ["Basic", "Standard", "Advanced"]
list = Listbox(root, height=3, width=20, selectmode = SINGLE)
list.grid()
for item in choices:
    list.insert(END, item)

foo = IntVar()

# Create a variable to hold the checkbox state and set it to True or False by default based on the AlwaysResetExtensions variable
checkbox_var = BooleanVar(value=AlwaysResetExtensions)

# Create a checkbox and link it to the variable
l3 = Checkbutton(root, text="Reset Extensions", variable=checkbox_var)
l3.grid()

l4 = Label(root, text='Choose the application:' ,relief = FLAT, fg = 'gray20' , anchor = "w", font = ('Arial', 10, 'bold'), width = 25)
l4.grid()

rb1 = Radiobutton(root, text = "ArcGIS Pro", variable=foo, value=0, anchor = "w", width = 25, padx=25)
rb1.grid()

if (InstallPythonModules and PythonModulesToInstall):
    rb2 = Radiobutton(root, text = "Install the following modules:", variable=foo, value=1, anchor = "w", width = 25, padx=25)
    rb2.grid()
    for item in PythonModulesToInstall:
        l4 = Label(root, text="- " + item , justify=LEFT, anchor = "w", padx=45)
        l4.grid(sticky=W)

if AddFileExplorer:
    rb3 = Radiobutton(root, text = "File Explorer", variable=foo, value=2, anchor = "w", width = 25, padx=25)
    rb3.grid()

def view_selected():
    flavour = str("")
    item = int(list.curselection()[0])
    number = foo.get()
    if (number == 0):
        if (item == 0):
            #print("Setting Basic (Viewer) licensing...")
            set_reg(path, 'ARCPRO_AUTH_TYPE', "Viewer")
            set_reg(path, 'SOFTWARE_CLASS', "Viewer")
            set_reg(path, 'SOFTWARE_CLASS_FN', "Viewer")
            set_reg(path, 'SingleUse_Installed', "Viewer")
        if (item == 1):        
            #print("Setting Standard (Editor) licensing...")
            set_reg(path, 'ARCPRO_AUTH_TYPE', "Editor")
            set_reg(path, 'SOFTWARE_CLASS', "Editor")
            set_reg(path, 'SOFTWARE_CLASS_FN', "Editor")
            set_reg(path, 'SingleUse_Installed', "Editor")
        if (item == 2):        
            #print("Setting Advanced (Professional) licensing...")
            set_reg(path, 'ARCPRO_AUTH_TYPE', "Professional")
            set_reg(path, 'SOFTWARE_CLASS', "Professional")
            set_reg(path, 'SOFTWARE_CLASS_FN', "Professional")
            set_reg(path, 'SingleUse_Installed', "Professional")
        if checkbox_var.get():
            delete_all_reg_values(path + r"\Extensions")
        flavour = str(ArcGISProPath)
    if (InstallPythonModules and PythonModulesToInstall):
        if (number == 1):
            for item in PythonModulesToInstall:
                output = ""
                command = [sys.executable, '-m', 'pip', 'install', item]
                if (sys.version_info[0] == 2 or (sys.version_info[0] == 3 and sys.version_info[1] < 5)):
                    try:
                        output = subprocess.check_output(command,universal_newlines=True)
                    except subprocess.CalledProcessError as e:
                        output = e.output
                    except:
                        output = "Unexpected error"
                    #print(output)
                if ((sys.version_info[0] == 3 and sys.version_info[1] >= 5) or sys.version_info[0] > 3):
                    process = subprocess.run(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE, universal_newlines=True)
                    if(len(process.stdout.strip()) > 0):
                        output = process.stdout
                    if(len(process.stderr.strip()) > 0):
                        output = process.stderr
                    if(len(process.stdout.strip()) == 0 and len(process.stderr.strip()) == 0):
                        output = "Unexpected error"
                    #print(output)
    if AddFileExplorer:
        if (number == 2):        
            flavour = str("C:\Windows\explorer.exe /n ,::{20D04FE0-3AEA-1069-A2D8-08002B30309D}")


    if flavour.strip():
        if os.path.exists(workingfolder):
            # Change the current working Directory to a user specific folder before launching
            os.chdir(workingfolder)
            #if sys.version_info[0] == 3:
                # messagebox.showinfo("Working Folder", workingfolder)
            #else:
                # tkMessageBox.showinfo("Working Folder", workingfolder)
        subprocess.Popen(flavour)

btn = Button(root, text = "Launch Selection", command = view_selected, width = 20, font = ('Arial', 10, 'bold') )
btn.grid()

if AlwaysDefaultToBasic:
    list.selection_set(0,)
else:
    current = (get_reg(path, 'SOFTWARE_CLASS_FN'))
    if current:
        if (current == "Viewer"):
            list.selection_set(0,)
        if (current == "Editor"):
            list.selection_set(1,)
        if (current == "Professional"):
            list.selection_set(2,)
        else:
            list.selection_set(0,)

if (MinutesBeforeClosing != 0):
    MinutesBeforeClosingText = StringVar()
    MinutesBeforeClosingText.set('after ' + str(MinutesBeforeClosing) + ' minutes.')
    l4 = Label(root, text= 'This window will automatically close' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l4.grid()
    l5 = Label(root, textvariable = MinutesBeforeClosingText ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l5.grid()

# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# According to your system settings such as fonts and screen resolution, 
# you might need to adjust the root.geometry(width,height). It's typically
# the height that becomes a problem with higher resolution (4K) screens.
# +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

# Getting screen's height in pixels
height = root.winfo_screenheight()
# Getting screen's width in pixels
width = root.winfo_screenwidth()
if EnableScreenResolutionDebugging:
    ScreenResolution = StringVar()
    ScreenResolution.set('width x height = ' + str(width) + ' x ' + str(height) + ' (in pixels)')
    l6 = Label(root, text='Screen Resolution:' ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l6.grid()
    l7 = Label(root, textvariable = ScreenResolution ,relief = FLAT, fg = 'gray60' , anchor = "w", font = ('Arial', 8, 'bold'))
    l7.grid()

height = str("220")
if AddFileExplorer:
    height = str(int(height) + 25)
if (InstallPythonModules and PythonModulesToInstall):
    height = str(int(height) + ((len(PythonModulesToInstall) + 1) * 25))
if (MinutesBeforeClosing != 0):
    height = str(int(height) + 50)
if EnableScreenResolutionDebugging:
    height = str(int(height) + 50)
root.geometry("315x" + height)

# Automatically close the window after a specified time
mytimer = int(MinutesBeforeClosing * 60 * 1000)
if (MinutesBeforeClosing != 0):
    close_after(mytimer)

root.mainloop()
